#include "../Framework/Framework.h"

#include "Poly.h"

class MainApp : public OpenGLApp {
protected:
	TextureID brick, floor, wall;
	TextureID renderTexture;

	bool freeFly;

	Set <Poly> polys;

	void drawWorld();
public:
	void selectPixelFormat(PixelFormat &pixelFormat);

	bool load();
	bool unload();
	bool drawFrame();
	
	void setKey(UINT nKey, bool value);
};

void MainApp::setKey(UINT nKey, bool value){
	OpenGLApp::setKey(nKey, value);

	if (nKey == demoModeKey && value) freeFly = !freeFly;
}

void MainApp::selectPixelFormat(PixelFormat &pixelFormat){
	pixelFormat.redBits   = 8;
	pixelFormat.greenBits = 8;
	pixelFormat.blueBits  = 8;
	pixelFormat.alphaBits = 0;

	pixelFormat.depthBits   = 24;
	pixelFormat.stencilBits = 0;
}

bool MainApp::load(){

	String supported, missing;
	if (GL_ATI_fragment_shader)    supported += "GL_ATI_fragment_shader\n";    else missing += "GL_ATI_fragment_shader\n";
    if (GL_EXT_texture_edge_clamp) supported += "GL_EXT_texture_edge_clamp\n"; else missing += "GL_EXT_texture_edge_clamp\n";
#ifdef WIN32
	if (WGL_ARB_render_texture)    supported += "WGL_ARB_render_texture\n";    else missing += "WGL_ARB_render_texture\n";
#else
	if (GLX_ATI_render_texture)    supported += "GLX_ATI_render_texture\n";    else missing += "GLX_ATI_render_texture\n";
#endif

    if (missing.getLength() > 0){
        ALERT("The following required OpenGL extensions are supported:\n\n" + supported +
				   "\nToo bad, these required OpenGL extension aren't supported by the graphic card and driver you're using:\n\n" + missing +
                   "\nThis application will now exit.", "Missing extensions");
        return false;
    }


	brick     = renderer->addTexture("../Textures/HumusBrick.dds");
	floor     = renderer->addTexture("../Textures/brickfloor.dds");
	wall      = renderer->addTexture("../Textures/wallpaper20.dds");

	textTexture  = renderer->addTexture("../Textures/font2.dds");

	renderTexture = renderer->addRenderTexture(512, 512, false, false);


	GLuint shaderID = glGenFragmentShadersATI(1);
	glBindFragmentShaderATI(shaderID);
   	glBeginFragmentShaderATI();
		GLfloat gray[] = {0.30f, 0.59f, 0.11f, 0.0f};
		glSetFragmentShaderConstantATI(GL_CON_0_ATI, gray);

		glSampleMapATI(GL_REG_0_ATI, GL_TEXTURE0_ARB, GL_SWIZZLE_STQ_ATI);
		glSampleMapATI(GL_REG_1_ATI, GL_TEXTURE1_ARB, GL_SWIZZLE_STQ_ATI);
		glSampleMapATI(GL_REG_2_ATI, GL_TEXTURE2_ARB, GL_SWIZZLE_STQ_ATI);
		glSampleMapATI(GL_REG_3_ATI, GL_TEXTURE3_ARB, GL_SWIZZLE_STQ_ATI);
		glSampleMapATI(GL_REG_4_ATI, GL_TEXTURE4_ARB, GL_SWIZZLE_STQ_ATI);

		glColorFragmentOp1ATI(GL_MOV_ATI, GL_REG_0_ATI, GL_NONE, GL_4X_BIT_ATI,
				                          GL_REG_0_ATI, GL_NONE, GL_NONE);

		glColorFragmentOp2ATI(GL_SUB_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
				                          GL_REG_0_ATI, GL_NONE, GL_NONE,
										  GL_REG_1_ATI, GL_NONE, GL_NONE);

		glColorFragmentOp2ATI(GL_SUB_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
				                          GL_REG_0_ATI, GL_NONE, GL_NONE,
					                      GL_REG_2_ATI, GL_NONE, GL_NONE);

		glColorFragmentOp2ATI(GL_SUB_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
				                          GL_REG_0_ATI, GL_NONE, GL_NONE,
					                      GL_REG_3_ATI, GL_NONE, GL_NONE);

		glColorFragmentOp2ATI(GL_SUB_ATI, GL_REG_0_ATI, GL_NONE, GL_4X_BIT_ATI,
				                          GL_REG_0_ATI, GL_NONE, GL_NONE,
					                      GL_REG_4_ATI, GL_NONE, GL_NONE);

		glColorFragmentOp2ATI(GL_DOT3_ATI, GL_REG_0_ATI, GL_NONE, GL_SATURATE_BIT_ATI,
				                           GL_REG_0_ATI, GL_NONE, GL_NONE,
					                       GL_CON_0_ATI, GL_NONE, GL_NONE);

		glColorFragmentOp2ATI(GL_MUL_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
				                          GL_REG_0_ATI, GL_NONE, GL_NONE,
					                      GL_REG_0_ATI, GL_NONE, GL_NONE);

		glColorFragmentOp2ATI(GL_MUL_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
				                          GL_REG_0_ATI, GL_NONE, GL_NONE,
					                      GL_REG_0_ATI, GL_NONE, GL_NONE);
	glEndFragmentShaderATI();
	renderer->addFragmentShader(GL_FRAGMENT_SHADER_ATI, shaderID);


 


	projection.loadProjectionMatrix(90 * PI / 180, 0.75f, 0.1f, 10000);

	polys.add(Poly(Vertex(-256, -100,  256), Vertex( 256, -100,  256), Vertex(-256, -100, -256), brick, 1, 1));
	polys.add(Poly(Vertex(-256, -100, -256), Vertex( 256, -100, -256), Vertex(-256, -256, -256), brick, 1, 0.25f));
	polys.add(Poly(Vertex( 256, -100, -256), Vertex( 256, -100,  256), Vertex( 256, -256, -256), brick, 1, 0.25f));
	polys.add(Poly(Vertex( 256, -100,  256), Vertex(-256, -100,  256), Vertex( 256, -256,  256), brick, 1, 0.25f));
	polys.add(Poly(Vertex(-256, -100,  256), Vertex(-256, -100, -256), Vertex(-256, -256,  256), brick, 1, 0.25f));

	polys.add(Poly(Vertex(-70,  70, -70), Vertex( 70,  70, -70), Vertex(-70, -70, -70), floor, 1, 1));
	polys.add(Poly(Vertex( 70,  70, -70), Vertex( 70,  70,  70), Vertex( 70, -70, -70), floor, 1, 1));
	polys.add(Poly(Vertex( 70,  70,  70), Vertex(-70,  70,  70), Vertex( 70, -70,  70), floor, 1, 1));
	polys.add(Poly(Vertex(-70,  70,  70), Vertex(-70,  70, -70), Vertex(-70, -70,  70), floor, 1, 1));
	polys.add(Poly(Vertex(-70,  70,  70), Vertex( 70,  70,  70), Vertex(-70,  70, -70), floor, 1, 1));
	polys.add(Poly(Vertex(-70, -70, -70), Vertex( 70, -70, -70), Vertex(-70, -70,  70), floor, 1, 1));

	
	polys.add(Poly(Vertex(-512, -256,  512), Vertex( 512, -256,  512), Vertex(-512, -256,  256), floor, 4, 1));
	polys.add(Poly(Vertex(-512, -256, -256), Vertex( 512, -256, -256), Vertex(-512, -256, -512), floor, 4, 1));
	polys.add(Poly(Vertex(-512, -256,  256), Vertex(-256, -256,  256), Vertex(-512, -256, -256), floor, 1, 2));
	polys.add(Poly(Vertex( 256, -256,  256), Vertex( 512, -256,  256), Vertex( 256, -256, -256), floor, 1, 2));

	polys.add(Poly(Vertex(-512,  512,  512), Vertex( 512,  512,  512), Vertex(-512, -256,  512), wall, 3, 3));
	polys.add(Poly(Vertex(-512,  512, -512), Vertex( 512,  512, -512), Vertex(-512,  512,  512), wall, 3, 3));
	polys.add(Poly(Vertex( 512,  512,  512), Vertex( 512,  512, -512), Vertex( 512, -256,  512), wall, 3, 3));
	polys.add(Poly(Vertex( 512,  512, -512), Vertex(-512,  512, -512), Vertex( 512, -256, -512), wall, 3, 3));
	polys.add(Poly(Vertex(-512,  512, -512), Vertex(-512,  512,  512), Vertex(-512, -256, -512), wall, 3, 3));

	return true;
}

bool MainApp::unload(){
	polys.clear();

	return true;
}

void MainApp::drawWorld(){
	glEnable(GL_CULL_FACE);
	glCullFace(GL_FRONT);
	for (unsigned int i = 0; i < polys.getSize(); i++){
		polys[i].drawBase(renderer);
	}
	glDisable(GL_CULL_FACE);
}


bool MainApp::drawFrame(){
	if (!freeFly){
		Vertex vert(15 * sinf(currTime * 2.0f), 15 * sinf(currTime * 3.24f) + 14, 35 * cosf(currTime * 0.73f));
		vert.normalize();
		vert *= 100 * (sinf(currTime + 0.43f) + 3.0f);

		Wx = atan2f(vert.y, sqrtf(vert.x * vert.x + vert.y * vert.y));
		Wy = atan2f(vert.x, vert.z);

		position = vert;
	}


	renderer->changeRenderTarget(renderTexture);
	
		glMatrixMode(GL_PROJECTION);
		glLoadMatrixf(projection);
		
		modelView.loadRotateZXY(-Wx, -Wy, -Wz);
		modelView.translate(-position);
		
		glMatrixMode(GL_MODELVIEW);
		glLoadMatrixf(modelView);
		
		glClear(/*GL_COLOR_BUFFER_BIT |*/ GL_DEPTH_BUFFER_BIT);
		
		glDepthFunc(GL_LEQUAL);
		glEnable(GL_DEPTH_TEST);
		
		drawWorld();

	renderer->changeRenderTarget();



	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,1,0,1,1,-1);

	renderer->setTextures(renderTexture, renderTexture, renderTexture, renderTexture, renderTexture);
	renderer->setFragmentShader(0);
	renderer->apply();

	float off = 1.0f / 256.0f;

	glBegin(GL_QUADS);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, off,     off);
		glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0,       off);
		glMultiTexCoord2fARB(GL_TEXTURE2_ARB, off * 2, off);
		glMultiTexCoord2fARB(GL_TEXTURE3_ARB, off,     0);
		glMultiTexCoord2fARB(GL_TEXTURE4_ARB, off,     off * 2);
		glVertex2f(0,0);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1 - off,     off);
		glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1 - off * 2, off);
		glMultiTexCoord2fARB(GL_TEXTURE2_ARB, 1,           off);
		glMultiTexCoord2fARB(GL_TEXTURE3_ARB, 1 - off,     0);
		glMultiTexCoord2fARB(GL_TEXTURE4_ARB, 1 - off,     off * 2);
		glVertex2f(1,0);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1 - off,     1 - off);
		glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1 - off * 2, 1 - off);
		glMultiTexCoord2fARB(GL_TEXTURE2_ARB, 1,           1 - off);
		glMultiTexCoord2fARB(GL_TEXTURE3_ARB, 1 - off,     1 - off * 2);
		glMultiTexCoord2fARB(GL_TEXTURE4_ARB, 1 - off,     1);
		glVertex2f(1,1);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, off,     1 - off);
		glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0,       1 - off);
		glMultiTexCoord2fARB(GL_TEXTURE2_ARB, off * 2, 1 - off);
		glMultiTexCoord2fARB(GL_TEXTURE3_ARB, off,     1 - off * 2);
		glMultiTexCoord2fARB(GL_TEXTURE4_ARB, off,     1);
		glVertex2f(0,1);
	glEnd();

	return true;
}

DECLARE_MAIN_APP(MainApp)
